(* Copyright 2010 Sarah Winkler
 * GNU Lesser General Public License
 *
 * This file is part of MKBtt.
 * 
 * MKBtt is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * MKBtt is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with MKBtt. If not, see <http://www.gnu.org/licenses/>.
 *)

(** Some auxiliary functions to deal with node sets.
@author Sarah Winkler
@since  2009/01/16 *)


(*** OPENS ***************************************************************)
open Util;;
(*** SUBMODULES **********************************************************)
module Rule = U.Rule;;
module Trs = U.Trs;;
module W = World;;
module Eq = Equation;;
module NS = IndexedSet;;
module CP = CompletionProcessx;;
module Nx = Nodex;;
module N = IndexedNode;;
module Monad = W.Monad;;

open Monad;;

(*** FUNCTIONS ***********************************************************)

(***** Projections *****)
let project = NS.map_union

let project_e p = project (Nodex.project_e p)

let project_r p ns = 
 project (Nodex.project_r p) ns >>= fun trs -> return (Trs.of_list trs)
;;

let project_c p ns =  
 project (Nodex.project_c p) ns >>= fun trs -> return (Trs.of_list trs)
;;
 
let project_r_closed p n =
 project (Nodex.project_r_closed p) n >>= fun trs -> 
 return (Trs.of_list trs) 
;;

let project_r_closed_unprotected p n =
 project (Nodex.project_r_closed_unprotected p) n >>= fun trs ->
 return (Trs.of_list trs)
;;

let project_e_closed p = project (Nodex.project_e_closed p)

(*
let nodeset_trs_union f ns =
 NS.fold
  (fun n trs -> let r = f n in Trs.union r trs)
   ns
   Trs.empty
;;

let nodeset_set_union f ns =
 NS.fold
  (fun n trs -> let r = f n in Setx.union r trs)
   ns
   (Setx.empty ())
;;*)

(***** Create initial nodes from equations *****)


let of_axioms = 
 let norm = (Monad.project Nx.s_normalize) <.> Eq.terms in
 let add xs x = 
  norm x >>= fun (s,t) ->
   if s <> t then 
    (N.create_axiom <.> (Util.uncurry Eq.of_terms)) (s,t) >>=
    N.id >>= fun i -> return (i::xs)
   else
    return xs
 in foldl add []
;;

let of_s_theory =
 W.get_options >>= (return <.> Completion.s_theory) >>= fun s ->
 let norm rl =
  let l,r = Rule.to_terms rl in 
  W.M.Equation.oriented_of_terms l r >>= fun (e,b) ->
  let l,r = if b then Eq.terms e else Pair.flip (Eq.terms e) in
  return (Rule.of_terms l r)
 in
 let create x = norm x >>= N.create_s_axiom in
 let add l x = create x >>= N.id >>= fun i -> N.close i >> return (i::l) in
 foldl add [] (Trs.to_list s) 
;;

(***** Count rules and equations in node set *****)
(*
let count ns p =
 let r = nodeset_trs_union (Node.project_all_rule p) ns in
 let e = nodeset_set_union (Node.project_all_eqn p) ns in
 let r' = Trs.to_set r in
 (NS.cardinal r') +  (NS.cardinal e)
;;*)

(***** Union of labels in nodes set *****)
(*
let label_union f nodeset =
 NS.fold
  (fun n pset -> CP.union (f n) pset)
  nodeset
  (CProcess.empty ())
;;

(***** Delete a process p in all labels of node set *****)
let delete p nset =
 CompletionProcess.delete p;
 NS.map (Node.delete_process p) nset
;;*)

(***** Check whether process occurs in some label of node set *****)
let contains_process_open p = NS.exists (Nodex.contains_process_open p)

(***** clean up nodes - filter out processes that are not open *****)
(*
let filter_alive nset1 nset2 =
 Setx.fold 
  (fun p (nset1', nset2') -> 
    if (contains_process_open p nset1') || (contains_process_open p nset2')
     then (nset1', nset2')
     else (delete p nset1', nset2')
  )
  !(CompletionProcess.all_processes)
  (nset1, nset2)
;;*)

(***** convert node set to string *****)
let to_string ns =
 NS.fold (fun n s -> (Nodex.to_string n) ^ ", " ^ s) ns ""
;;
