
(*** OPENS ***************************************************************)
open Util;;

(*** SUBMODULES **********************************************************)
module Var = Rewriting.Variable;;
module Fun = Rewriting.Function;;
module M = U.Monad;;
module L = U.Label;;
module T = U.Term;;
module R = U.Rule;;
module Trs = U.Trs;;

(*** TYPES ***************************************************************)
type symbol_theory =
 | AC of Fun.t
 | ACU of (Fun.t * Fun.t) (* f, zero *)

type t = symbol_theory list

(*** OPENS (2) ***********************************************************)
open M;;

(*** FUNCTIONS ***********************************************************)
let acu = 
 fresh_var >>= fun x ->
 M.fresh_fun >>= M.create_fun_name >>= (M.create_fun 2) >>= fun f ->
 M.fresh_fun >>= M.create_fun_name >>= (M.create_fun 0) >>= fun z ->
 let r1 = R.of_terms (T.Fun(f,[T.Var x; T.Fun(z,[])])) (T.Var x) in
 let r2 = R.of_terms (T.Fun(f,[T.Fun(z,[]);T.Var x])) (T.Var x) in
 return (r1,r2)
;;

let acu_variant rl =
 match R.to_terms rl with
  | (T.Fun(f,[T.Var x; T.Fun(z,[])]), T.Var x') when x=x' -> true
  | (T.Fun(f,[T.Fun(z,[]); T.Var x]), T.Var x') when x=x' -> true
  | _ -> false
;;

let acu_symbols rl =
 match R.to_terms rl with
  | (T.Fun(f,[T.Var x; T.Fun(z,[])]), T.Var x') when x=x' -> (f,z)
  | (T.Fun(f,[T.Fun(z,[]); T.Var x]), T.Var x') when x=x' -> (f,z)
  | _ -> failwith "no acu rule"
;;

let recognize_acu trs =
 acu >>= fun (r1,r2) ->
 match
  (try Some (List.find acu_variant (Trs.to_list trs)) 
   with Not_found -> None)
 with
  | None -> return []
  | Some rl ->
   let f,z = acu_symbols rl in
   M.is_theory L.AC f >>= fun f_is_ac ->
   if f_is_ac then return [ACU(f,z)] else return []
;;

let is_acu_symbol f = function
 [] -> false
 | (ACU (f',_)) :: _ when f = f' -> true
 | _ -> false
;;

let unit_for f = function
 [] -> failwith "no unit exists" 
 | (ACU (f',z)) :: _ when f = f' -> z
 | _ -> failwith "unexpected pattern in unit_for"
;;
